@@ -6,12 +6,13 @@ import json
 import logging
 import os
 import sys
+from typing import Iterator, Sequence, Optional, Tuple, List
 
 from jsonschema.exceptions import ValidationError
 
 import cfnlint.runner
 from cfnlint.template import Template
-from cfnlint.rules import RulesCollection, ParseError, TransformError
+from cfnlint.rules import Match, RulesCollection, ParseError, TransformError
 import cfnlint.config
 import cfnlint.formatters
 import cfnlint.decode
@@ -22,12 +23,19 @@ LOGGER = logging.getLogger('cfnlint')
 DEFAULT_RULESDIR = os.path.join(os.path.dirname(__file__), 'rules')
 __CACHED_RULES = None
 
+Matches = List[Match]
+RulesCollectionNone = Optional[RulesCollection]
+ArgsFilename = Tuple[cfnlint.config.ConfigMixIn,
+                     List[Optional[str]], cfnlint.formatters.BaseFormatter]
+TemplateRules = Tuple[Optional[str], RulesCollectionNone, Optional[Matches]]
+
+
 class CfnLintExitException(Exception):
     """Generic exception used when the cli should exit"""
 
     def __init__(self, msg=None, exit_code=1):
         if msg is None:
-            msg = 'process failed with exit code %s' % exit_code
+            msg = f'process failed with exit code {exit_code}'
         super(CfnLintExitException, self).__init__(msg)
         self.exit_code = exit_code
 
@@ -40,10 +48,17 @@ class UnexpectedRuleException(CfnLintExitException):
     """When processing a rule fails in an unexpected way"""
 
 
-def run_cli(filename, template, rules, regions, override_spec, build_graph, registry_schemas, mandatory_rules=None):
+def run_cli(filename: str,
+            template: str,
+            rules: RulesCollectionNone,
+            regions: Sequence[str],
+            override_spec: dict,
+            build_graph: bool,
+            registry_schemas: Sequence[str],
+            mandatory_rules: Optional[Sequence[str]] = None
+            ) -> Matches:
     """Process args and run"""
 
-
     if override_spec:
         cfnlint.helpers.override_specs(override_spec)
 
@@ -61,7 +76,7 @@ def run_cli(filename, template, rules, regions, override_spec, build_graph, regi
     return run_checks(filename, template, rules, regions, mandatory_rules)
 
 
-def get_exit_code(matches):
+def get_exit_code(matches: Matches) -> int:
     """ Determine exit code """
     exit_code = 0
     for match in matches:
@@ -74,34 +89,39 @@ def get_exit_code(matches):
 
     return exit_code
 
+# pylint: disable=too-many-return-statements
 
-def get_formatter(fmt):
-    formatter = {}
+
+def get_formatter(fmt: str) -> cfnlint.formatters.BaseFormatter:
     if fmt:
         if fmt == 'quiet':
-            formatter = cfnlint.formatters.QuietFormatter()
-        elif fmt == 'parseable':
+            return cfnlint.formatters.QuietFormatter()
+        if fmt == 'parseable':
             # pylint: disable=bad-option-value
-            formatter = cfnlint.formatters.ParseableFormatter()
-        elif fmt == 'json':
-            formatter = cfnlint.formatters.JsonFormatter()
-        elif fmt == 'junit':
-            formatter = cfnlint.formatters.JUnitFormatter()
-        elif fmt == 'pretty':
-            formatter = cfnlint.formatters.PrettyFormatter()
-        elif fmt == 'sarif':
-            formatter = cfnlint.formatters.SARIFFormatter()
-    else:
-        formatter = cfnlint.formatters.Formatter()
+            return cfnlint.formatters.ParseableFormatter()
+        if fmt == 'json':
+            return cfnlint.formatters.JsonFormatter()
+        if fmt == 'junit':
+            return cfnlint.formatters.JUnitFormatter()
+        if fmt == 'pretty':
+            return cfnlint.formatters.PrettyFormatter()
+        if fmt == 'sarif':
+            return cfnlint.formatters.SARIFFormatter()
 
-    return formatter
+    return cfnlint.formatters.Formatter()
 
 
-def get_rules(append_rules, ignore_rules, include_rules, configure_rules=None, include_experimental=False,
-              mandatory_rules=None, custom_rules=None):
+def get_rules(append_rules: List[str],
+              ignore_rules: Sequence[str],
+              include_rules: Sequence[str],
+              configure_rules=None,
+              include_experimental: bool = False,
+              mandatory_rules: Sequence[str] = None,
+              custom_rules: str = None
+              ) -> RulesCollection:
     rules = RulesCollection(ignore_rules, include_rules, configure_rules,
                             include_experimental, mandatory_rules)
-    rules_paths = [DEFAULT_RULESDIR] + append_rules
+    rules_paths: List[str] = [DEFAULT_RULESDIR] + append_rules
     try:
         for rules_path in rules_paths:
             if rules_path and os.path.isdir(os.path.expanduser(rules_path)):
@@ -111,33 +131,37 @@ def get_rules(append_rules, ignore_rules, include_rules, configure_rules=None, i
 
         rules.create_from_custom_rules_file(custom_rules)
     except (OSError, ImportError) as e:
-        raise UnexpectedRuleException('Tried to append rules but got an error: %s' % str(e), 1)
+        raise UnexpectedRuleException(
+            f'Tried to append rules but got an error: {str(e)}', 1)
     return rules
 
 
-def get_matches(filenames, args):
+def get_matches(filenames: str, args: cfnlint.config.ConfigMixIn) -> Iterator[Match]:
     for filename in filenames:
         LOGGER.debug('Begin linting of file: %s', str(filename))
         (template, rules, errors) = get_template_rules(filename, args)
         # template matches may be empty but the template is still None
         # this happens when ignoring bad templates
         if not errors and template:
-            matches = run_cli(filename, template, rules, args.regions, args.override_spec, args.build_graph, args.registry_schemas, args.mandatory_checks)
+            matches = run_cli(filename, template, rules, args.regions, args.override_spec,
+                              args.build_graph, args.registry_schemas, args.mandatory_checks)
             for match in matches:
                 yield match
         else:
-            for match in errors:
-                yield match
+            if errors:
+                for match in errors:
+                    yield match
         LOGGER.debug('Completed linting of file: %s', str(filename))
 
 
 def configure_logging(debug_logging):
     """ Backwards compatibility for integrators """
-    LOGGER.info('Update your integrations to use "cfnlint.config.configure_logging" instead')
+    LOGGER.info(
+        'Update your integrations to use "cfnlint.config.configure_logging" instead')
     cfnlint.config.configure_logging(debug_logging, False)
 
 
-def get_args_filenames(cli_args):
+def get_args_filenames(cli_args: Sequence[str]) -> ArgsFilename:
     """ Get Template Configuration items and set them as default values"""
     try:
         config = cfnlint.config.ConfigMixIn(cli_args)
@@ -149,12 +173,13 @@ def get_args_filenames(cli_args):
     formatter = get_formatter(fmt)
 
     if config.update_specs:
-        cfnlint.maintenance.update_resource_specs()
+        cfnlint.maintenance.update_resource_specs(config.force)
         sys.exit(0)
 
     if config.update_documentation:
         # Get ALL rules (ignore the CLI settings))
-        documentation_rules = cfnlint.core.get_rules([], [], ['I', 'E', 'W'], {}, True, [])
+        documentation_rules = cfnlint.core.get_rules(
+            [], [], ['I', 'E', 'W'], {}, True, [])
         cfnlint.maintenance.update_documentation(documentation_rules)
         sys.exit(0)
 
@@ -183,20 +208,19 @@ def get_args_filenames(cli_args):
 
     return(config, config.templates, formatter)
 
-
-def get_used_rules():
+def get_used_rules() -> Optional[RulesCollection]:
     return __CACHED_RULES
 
-def _reset_rule_cache():
+def _reset_rule_cache() -> None:
     """ Reset the rule cache. Used mostly for testing"""
     global __CACHED_RULES #pylint: disable=global-statement
     __CACHED_RULES = None
 
-def get_template_rules(filename, args):
+def get_template_rules(filename: str, args: cfnlint.config.ConfigMixIn) -> TemplateRules:
     """ Get Template Configuration items and set them as default values"""
-    global __CACHED_RULES  #pylint: disable=global-statement
+    global __CACHED_RULES  # pylint: disable=global-statement
 
-    ignore_bad_template = False
+    ignore_bad_template: bool = False
     if args.ignore_bad_template:
         ignore_bad_template = True
     else:
@@ -214,8 +238,8 @@ def get_template_rules(filename, args):
 
     if errors:
         if len(errors) == 1 and ignore_bad_template and errors[0].rule.id == 'E0000':
-            return(template, [], [])
-        return(template, [], errors)
+            return(template, None, [])
+        return(template, None, errors)
 
     args.template_args = template
 
@@ -241,23 +265,29 @@ def get_template_rules(filename, args):
     return(template, __CACHED_RULES, [])
 
 
-def run_checks(filename, template, rules, regions, mandatory_rules=None):
+def run_checks(filename: str, template: str,
+               rules: RulesCollectionNone,
+               regions: Sequence[str],
+               mandatory_rules: Optional[Sequence[str]] = None
+               ) -> Matches:
     """Run Checks and Custom Rules against the template"""
     if regions:
         if not set(regions).issubset(set(REGIONS)):
             unsupported_regions = list(set(regions).difference(set(REGIONS)))
-            msg = 'Regions %s are unsupported. Supported regions are %s' % (
-                unsupported_regions, REGIONS)
+            msg = f'Regions {unsupported_regions} are unsupported. Supported regions are {REGIONS}'
             raise InvalidRegionException(msg, 32)
 
-    errors = []
+    errors: Matches = []
+
+    if not isinstance(rules, RulesCollection):
+        return ([])
 
     runner = cfnlint.runner.Runner(rules, filename, template, regions,
                                    mandatory_rules=mandatory_rules)
 
     # Transform logic helps with handling serverless templates
-    ignore_transform_error = False
-    if not rules.is_rule_enabled(TransformError()):
+    ignore_transform_error: bool = False
+    if isinstance(rules, RulesCollection) and not rules.is_rule_enabled(TransformError()):
         ignore_transform_error = True
 
     errors.extend(runner.transform())
@@ -272,7 +302,7 @@ def run_checks(filename, template, rules, regions, mandatory_rules=None):
     try:
         errors.extend(runner.run())
     except Exception as err:  # pylint: disable=W0703
-        msg = 'Tried to process rules on file %s but got an error: %s' % (filename, str(err))
+        msg = f'Tried to process rules on file {filename} but got an error: {str(err)}'
         UnexpectedRuleException(msg, 1)
     errors.sort(key=lambda x: (x.filename, x.linenumber, x.rule.id))
 
